%  POLYFIT Fit Polynomial To Data
%  
%  
%  USAGE
%  
%  The polyfit routine has the following syntax
%  
%    p = polyfit(x,y,n)
%  
%  where x and y are vectors of the same size, and
%  n is the degree of the approximating polynomial.  
%  The resulting vector p forms the coefficients of
%  the optimal polynomial (in descending degree) that fit
%  y with x.  
%  
%  The polyfit routine finds the approximating polynomial
%  
%  $
%     p(x) = p_1 x^n + p_2 x^{n-1} + \dots + p_n x + p_{n+1}
%  $
%  
%  such that
%  
%  $
%     \sum_{i} (p(x_i) - y_i)^2
%  $
%  
%  is minimized.  It does so by forming the Vandermonde matrix
%  and solving the resulting set of equations using the backslash
%  operator.  Note that the Vandermonde matrix can become poorly
%  conditioned with large n quite rapidly.
%  
function p = polyfit(x,y,n)
x = x(:);
y = y(:);
if (size(x,1) ~= size(y,1))
  error('size mismatch on arguments to polyfit - they should be the same size');
end
if (n < 0 | n > 200)
  error('invalid polynomial order in polyfit - must be between [1,200] inclusive');
end
A = zeros(size(x,1),n+1);
xp = x*0.0f+1.0f;
for i=(n+1):-1:1
  A(:,i) = xp;
  xp = xp.*x;
end
p = (A\y)';
